# this script test the thread-safetness of the pgpy database adapter.
# we need a clean database and a user and password for the connection,
# so, please, modify the lines below as you need.

DSN = 'dbname=test user=test'
TABLE = 'casttest'

#### there should be no need to modify anything below this line

import psycopg, threading, whrandom
from psycopg import ProgrammingError


#### a class used to store rectangles and a production function

class Rect:
    def __init__(self, x0, y0, x1, y1):
        self.x0 = x0
        self.x1 = x1
        self.y0 = y0
        self.y1 = y1
    def __repr__(self):
        return "'((%d, %d), (%d, %d))'" % (self.x0, self.y0, self.x1, self.y1)
    def show(self):
        return "I am a nice box of width %d and height %d" % \
               (abs(self.x1-self.x0), abs(self.y1-self.y0))

def makeRect(s):
    t = eval('('+s+')')
    return Rect(t[0][0], t[0][1], t[1][0], t[1][1])
    
# obtain a connection and obtain a cursor to create the tables
o = psycopg.connect(DSN)
c = o.cursor()

# create the table
create = 'CREATE TABLE %s (p1 point, p2 point, b box)' % TABLE

try:
    print "Creating table", TABLE
    c.execute(create)
except ProgrammingError:
    print "Warning:", TABLE, "already exists, dropping and re-creating"
    c.execute('DROP TABLE %s' % TABLE)
    c.execute(create)
c.commit()

# select nothing only to get column description
c.execute("SELECT b FROM %s" % TABLE)
boxoid = c.description[0][1]

# and build user type object
BOX = psycopg.new_type((boxoid,), "BOX", makeRect)
psycopg.register_type(BOX)

# now insert 100 random data (2 points and a box)
for i in range(100):
    p1 = (whrandom.randint(0,100), whrandom.randint(0,100))
    p2 = (whrandom.randint(0,100), whrandom.randint(0,100))
    b = Rect(whrandom.randint(0,100), whrandom.randint(0,100),
             whrandom.randint(0,100), whrandom.randint(0,100))
    c.execute("INSERT INTO %(table)s VALUES ('%(p1)s', '%(p2)s', %(box)s)",
              {'box':b, 'p1':p1, 'p2':p2, 'table':TABLE})
c.commit()

# select and print box
c.execute("SELECT b FROM %s WHERE p1 @ b OR p2 @ b" % TABLE)
boxes = c.fetchall()
print "FOUND %d BOXES" % len(boxes)
for b in boxes:
    print b[0].show()
